#ifndef __LIBWIN_H__
#define __LIBWIN_H__


#include <stdio.h>
#include <Xlib.h>
#include <Xutil.h>
#include <mlvalues.h>

typedef struct {
  int w, h;             	 /* Dimensions of the drawable */
  Window win;                	 /* The drawable itself */
  GC gc; 			 /* The associated graphics context */
  int grx,gry;   		 /* Position of the current point */
  XFontStruct * grfont;          /* Current font */
  char underline;                /* Is current font drawing underline */
}canvas;

#define NbMaxWindow   64


extern Display * grdisplay;            /* The display connection */
extern int grscreen;                   /* The screen number */
extern int grscreen_height;            /* The screen height */
extern int grscreen_width;             /* The screen width */
extern Colormap grcolormap;            /* The color map */
extern canvas *grwindows;              /* The graphics window */
extern int nb_windows;                 /* The number of windows */
extern int current_window;	       /* The number of windows */
extern int grwhite, grblack;           /* Black and white pixels */
extern unsigned long grcolor;          /* Current drawing color */
extern char ValidWindow;               /* is current window valid */
extern Bool gr_initialized;
extern int gr_alarm_time;              /* the current alarm time */


extern char err_name[];
extern char out_name[];
extern char camlwin_env[];

#define Wcvt(y) (grwindows[current_window].h - 1 - (y))
#define Scvt(y) (grscreen_height - 1 - (y))
#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))

#define DEFAULT_SCREEN_WIDTH 600
#define DEFAULT_SCREEN_HEIGHT 450
#define BORDER_WIDTH 2
#define WINDOW_NAME "Caml Light windows"
#define ICON_NAME "Caml Light windows"
#define DEFAULT_EVENT_MASK \
   (ExposureMask | KeyPressMask | ButtonPressMask | ButtonReleaseMask | \
    PointerMotionMask | StructureNotifyMask | SubstructureNotifyMask)
#define DEFAULT_FONT "fixed"
#define SIZE_QUEUE 256
#define TRUE    1
#define FALSE   0

#define Button_down  0
#define Button_up    1
#define Keypressed   2
#define Motion       3
#define Poll         4
#define Redraw       5
#define Close        6
#define Size         7
#define Time         8


#define F1_code        0xFF80
#define F2_code        0xFF81
#define F3_code        0xFF82
#define F4_code        0xFF83
#define F5_code        0xFF84
#define F6_code        0xFF85
#define F7_code        0xFF86
#define F8_code        0xFF87
#define F9_code        0xFF88
#define F10_code       0xFF89

#define Right_code     0xFF00
#define Left_code      0xFF01
#define Up_code        0xFF02
#define Down_code      0xFF03

#define PageUp_code    0xFF04
#define PageDown_code  0xFF05
#define Home_code      0xFF06
#define End_code       0xFF07

#define Del_code       0xFF09
#define Ins_code       0xFF08

#define Copy_code      0xFF20
#define Cut_code       0xFF21
#define Past_code      0xFF22


void gr_fail();
void gr_check_open();
unsigned long gr_pixel_rgb();
int gr_rgb_pixel();
void gr_handle_simple_event();
void gr_enqueue_events();
value gr_add_window();
void gr_time_handler();

#endif
